@@ -4,7 +4,8 @@ import os
 import re
 import warnings
 from collections import OrderedDict
-from urllib.parse import urljoin, urlparse
+from html import escape, unescape
+from urllib.parse import urljoin, urlparse, unquote
 
 import cssutils
 import requests
@@ -27,8 +28,7 @@ class ExternalNotFoundError(ValueError):
 
 
 def make_important(bulk):
-    """makes every property in a string !important.
-    """
+    """makes every property in a string !important."""
     return ";".join(
         "%s !important" % p if not p.endswith("!important") else p
         for p in bulk.split(";")
@@ -36,8 +36,7 @@ def make_important(bulk):
 
 
 def get_or_create_head(root):
-    """Ensures that `root` contains a <head> element and returns it.
-    """
+    """Ensures that `root` contains a <head> element and returns it."""
     head = _create_cssselector("head")(root)
     if not head:
         head = etree.Element("head")
@@ -51,18 +50,18 @@ def get_or_create_head(root):
 @function_cache()
 def _cache_parse_css_string(css_body, validate=True):
     """
-        This function will cache the result from cssutils
-        It is a big gain when number of rules is big
-        Maximum cache entries are 1000. This is mainly for
-        protecting memory leak in case something gone wild.
-        Be aware that you can turn the cache off in Premailer
+    This function will cache the result from cssutils
+    It is a big gain when number of rules is big
+    Maximum cache entries are 1000. This is mainly for
+    protecting memory leak in case something gone wild.
+    Be aware that you can turn the cache off in Premailer
 
-        Args:
-            css_body(str): css rules in string format
-            validate(bool): if cssutils should validate
+    Args:
+        css_body(str): css rules in string format
+        validate(bool): if cssutils should validate
 
-        Returns:
-            cssutils.css.cssstylesheet.CSSStyleSheet
+    Returns:
+        cssutils.css.cssstylesheet.CSSStyleSheet
 
     """
     return cssutils.parseString(css_body, validate=validate)
@@ -74,8 +73,7 @@ def _create_cssselector(selector):
 
 
 def capitalize_float_margin(css_body):
-    """Capitalize float and margin CSS property names
-    """
+    """Capitalize float and margin CSS property names"""
 
     def _capitalize_property(match):
         return "{0}:{1}{2}".format(
@@ -116,6 +114,7 @@ class Premailer(object):
         disable_link_rewrites=False,
         preserve_internal_links=False,
         preserve_inline_attachments=True,
+        preserve_handlebar_syntax=False,
         exclude_pseudoclasses=True,
         keep_style_tags=False,
         include_star_selectors=False,
@@ -152,6 +151,7 @@ class Premailer(object):
         self.disable_link_rewrites = disable_link_rewrites
         self.preserve_internal_links = preserve_internal_links
         self.preserve_inline_attachments = preserve_inline_attachments
+        self.preserve_handlebar_syntax = preserve_handlebar_syntax
         self.exclude_pseudoclasses = exclude_pseudoclasses
         # whether to delete the <style> tag once it's been processed
         # this will always preserve the original css
@@ -206,7 +206,7 @@ class Premailer(object):
                 return "{0}:{1} !important".format(prop.name, prop.value)
 
         def join_css_properties(properties):
-            """ Accepts a list of cssutils Property objects and returns
+            """Accepts a list of cssutils Property objects and returns
             a semicolon delimitted string like 'color: red; font-size: 12px'
             """
             return ";".join(format_css_property(prop) for prop in properties)
@@ -309,6 +309,39 @@ class Premailer(object):
             else:
                 parser = etree.HTMLParser()
             stripped = html.strip()
+
+            # Escape all characters in handlebars in HTML attributes.
+            # Without this step, if handlebars were to include a character such as ",
+            # etree.fromstring() would not be able to differentiate the "'s in the value
+            # from the "'s for the attribute.
+            # --------------------------------------------------------------------------
+            # Provided the input below:
+            # <a href="{{ "<Test>" }}"></a>
+            # --------------------------------------------------------------------------
+            # Decoded result without preservation:
+            # <a href="%7B%7B%20">" }}"&gt;</a>
+            # Everything between the first two quotes were treated as the value of the
+            # attribute. Then, the characters between the second quote and the second
+            # > were treated as invalid attributes and discarded. Lastly, the value of
+            # the original attribute after the second and </a> were treated as the
+            # contents of the HTML tag.
+            # ---
+            # Result:
+            # <a href="%7B%7B%20">" }}"&gt;</a>
+            # --------------------------------------------------------------------------
+            # Decoded result with preservation (prior to unescape() & unquote()):
+            # <a href="%7B%7B%20%22&lt;Test&gt;%22%20%7D%7D"></a>
+            # No value was lost in the encoding process.
+            # ---
+            # Result after unquote() and unescape():
+            # <a href="{{ "<Test>" }}"></a>
+            if self.preserve_handlebar_syntax:
+                stripped = re.sub(
+                    r'="{{(.*?)}}"',
+                    lambda match: '="{{' + escape(match.groups()[0]) + '}}"',
+                    stripped,
+                )
+
             tree = etree.fromstring(stripped, parser).getroottree()
             page = tree.getroot()
             # lxml inserts a doctype if none exists, so only include it in
@@ -518,6 +551,14 @@ class Premailer(object):
                 out = _cdata_regex.sub(
                     lambda m: "/*<![CDATA[*/%s/*]]>*/" % m.group(1), out
                 )
+            # Replace %xx escapes and HTML entities, within handlebars in HTML
+            # attributes, with their single-character equivalents.
+            if self.preserve_handlebar_syntax:
+                out = re.sub(
+                    r'="%7B%7B(.+?)%7D%7D"',
+                    lambda match: '="{{' + unescape(unquote(match.groups()[0])) + '}}"',
+                    out,
+                )
             return out
 
     def _load_external_url(self, url):
@@ -526,8 +567,7 @@ class Premailer(object):
         return response.text
 
     def _load_external(self, url):
-        """loads an external stylesheet from a remote url or local path
-        """
+        """loads an external stylesheet from a remote url or local path"""
         if url.startswith("//"):
             # then we have to rely on the base_url
             if self.base_url and "https://" in self.base_url:
@@ -611,8 +651,7 @@ class Premailer(object):
             element.attrib[key] = value
 
     def _css_rules_to_string(self, rules):
-        """given a list of css rules returns a css string
-        """
+        """given a list of css rules returns a css string"""
         lines = []
         for item in rules:
             if isinstance(item, tuple):
